==> ikapi-2.0.0-alpha2.tgz: package/src/Database/DDB/IKDBQueryPut.js <==
import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryPut extends IKDBBaseQuery {
  constructor(indexName) {
    super(indexName);
    this.putItems = {};
  }
  /*public methods*/
  appendDefaultTrail(userID) {
    this.appendValue(userID, 'updatedBy');
    this.appendValue(Date.now(), 'updatedOn');
  }
  //return if the value was appended or not
  appendValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
       ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.putItems[key] = val;
      console.debug(`Append ${key}@${val}`);
      return true;
    } return false;
  }
  //dbKey and prefix are optional values
  appendObjectForKeyValue(source, key, dbKey, prefix) {
    //Check for optional db key that can differ from input key
    if (dbKey == undefined || dbKey == null) dbKey = key;
    //check if value is valid
    if (source[key] != undefined &&
       ((Object.keys(source[key]).length > 0) ||
        (!isNaN(source[key]) && source[key].length != 0) ||
        (source[key].length > 0))) {
      //Check for object conversion
      let val = (prefix != undefined && source[key].indexOf(prefix) == -1 ?
                  prefix + source[key] : source[key]);
      //valid
      if (val != undefined) { this.putItems[dbKey]= val; }
      console.debug(`Append-ofk ${dbKey}@${val}`);
      return true;
    } return false;
  }
  setItems(items) { this.putItems = items; }
  //
  async run(dbManager) {
    const resp = await this.rawRun(dbManager);
    if (resp.Items) {
      const unmarshalled = Utils.decapsulateForDB(resp.Items);
      console.debug('Parsed result: ', unmarshalled);
      return unmarshalled;
    } return (resp.ConsumedCapacity != undefined && resp.ConsumedCapacity.CapacityUnits != undefined);
  }
  async rawRun(dbManager) {
    const query = this._rawQuery(dbManager.tableName);
    console.log('Putting item: ', query);
    const resp = await dbManager.connection.putItem(query).promise();
    console.debug('Raw result: ', resp);
    return resp;
  }

  /*private methods*/
  _rawQuery(dbTableName, isTransaction) {
    return this._buildPutQuery({
                ...super._rawQuery(dbTableName, isTransaction),
                ...(isTransaction ? {} : {ReturnValues: "ALL_OLD" /*request new values added*/ })});
  }
  _buildPutQuery(expression) {
    if (Object.keys(this.putItems).length > 0) {
      if (!expression.Item) expression.Item = {};
      for (let itemKey of Object.keys(this.putItems)) {
        const item = this.putItems[itemKey];
        expression.Item[itemKey] = Utils.encapsulateForDB(item);
      }
    } return expression;
  }
}


/*marshalling should be done
let unMarshalled = dbResp.Items.map((item) => {
  let i = AWS.DynamoDB.Converter.unmarshall(item);
  const  { gsi1sk, gsi3pk,...otherProps } = i;
  const activitydate = gsi1sk;
  const ctid = gsi3pk;
  const ret = { activitydate, ctid, ...otherProps };
  console.log("um:", ret);
  return ret;
});
unMarshalled.request_data = AWS.DynamoDB.Converter.unmarshall(unMarshalled.request_data);
console.log(unMarshalled);
return unMarshalled;
*/

==> ikapi-3.0.0-alpha2.tgz: package/src/Database/DDB/IKDBQueryPut.js <==
import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryPut extends IKDBBaseQuery {
  constructor(indexName) {
    super(indexName);
    this.putItems = {};
  }
  /*public methods*/
  appendDefaultTrail(userID) {
    this.appendValue(userID, 'updatedBy');
    this.appendValue(Date.now(), 'updatedOn');
  }
  //return if the value was appended or not
  appendValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
       ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) != 0 ? prefix + val : val);
      //valid
      this.putItems[key] = val;
      // console.debug(`Append ${key}@${val}`);
      return true;
    } return false;
  }
  //dbKey and prefix are optional values
  appendObjectForKeyValue(source, key, dbKey, prefix) {
    //Check for optional db key that can differ from input key
    if (dbKey == undefined || dbKey == null) dbKey = key;
    //check if value is valid
    if (source[key] != undefined &&
       ((Object.keys(source[key]).length > 0) ||
        (!isNaN(source[key]) && source[key].length != 0) ||
        (source[key].length > 0))) {
      //Check for object conversion
      let val = (prefix != undefined && source[key].indexOf(prefix) != 0 ?
                  prefix + source[key] : source[key]);
      //valid
      if (val != undefined) { this.putItems[dbKey]= val; }
      // console.debug(`Append-ofk ${dbKey}@${val}`);
      return true;
    } return false;
  }
  setItems(items) { this.putItems = items; }
  //
  async run(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const resp = await this.rawRun(dbManager);
    if (resp.Items) {
      const unmarshalled = Utils.decapsulateForDB(resp.Items);
      localConsole.debug('Parsed result: ', unmarshalled);
      return unmarshalled;
    } return (resp.ConsumedCapacity != undefined && resp.ConsumedCapacity.CapacityUnits != undefined);
  }
  async rawRun(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const query = this._rawQuery(dbManager.tableName);
    localConsole.log('Putting item: ', query);
    const resp = await dbManager.connection.putItem(query).promise();
    // localConsole.debug('Raw result: ', resp);
    return resp;
  }

  /*private methods*/
  _rawQuery(dbTableName, isTransaction) {
    return this._buildPutQuery({
                ...super._rawQuery(dbTableName, isTransaction),
                ...(isTransaction ? {} : {ReturnValues: "ALL_OLD" /*request new values added*/ })});
  }
  _buildPutQuery(expression) {
    if (Object.keys(this.putItems).length > 0) {
      if (!expression.Item) expression.Item = {};
      for (let itemKey of Object.keys(this.putItems)) {
        const item = this.putItems[itemKey];
        expression.Item[itemKey] = Utils.encapsulateForDB(item);
      }
    } return expression;
  }
}
